Re: [PATCH v2 07/63] skbuff: Switch structure bounds to struct_group()

2021-09-01 Thread Jason A. Donenfeld
For the drivers/net/wireguard part,

Reviewed-by: Jason A. Donenfeld 


[PATCH v2 07/63] skbuff: Switch structure bounds to struct_group()

2021-08-18 Thread Kees Cook
In preparation for FORTIFY_SOURCE performing compile-time and run-time
field bounds checking for memcpy(), memmove(), and memset(), avoid
intentionally writing across neighboring fields.

Replace the existing empty member position markers "headers_start" and
"headers_end" with a struct_group(). This will allow memcpy() and sizeof()
to more easily reason about sizes, and improve readability.

"pahole" shows no size nor member offset changes to struct sk_buff.
"objdump -d" shows no object code changes (outside of WARNs affected by
source line number changes).

Cc: "Jason A. Donenfeld" 
Cc: "David S. Miller" 
Cc: Jakub Kicinski 
Cc: Jonathan Lemon 
Cc: Alexander Lobakin 
Cc: Jakub Sitnicki 
Cc: Marco Elver 
Cc: Willem de Bruijn 
Cc: wireguard@lists.zx2c4.com
Cc: net...@vger.kernel.org
Signed-off-by: Kees Cook 
Reviewed-by: Gustavo A. R. Silva 
Link: https://lore.kernel.org/lkml/20210728035006.GD35706@embeddedor
---
 drivers/net/wireguard/queueing.h |  4 +---
 include/linux/skbuff.h   |  9 -
 net/core/skbuff.c| 14 +-
 3 files changed, 10 insertions(+), 17 deletions(-)

diff --git a/drivers/net/wireguard/queueing.h b/drivers/net/wireguard/queueing.h
index 4ef2944a68bc..52da5e963003 100644
--- a/drivers/net/wireguard/queueing.h
+++ b/drivers/net/wireguard/queueing.h
@@ -79,9 +79,7 @@ static inline void wg_reset_packet(struct sk_buff *skb, bool 
encapsulating)
u8 sw_hash = skb->sw_hash;
u32 hash = skb->hash;
skb_scrub_packet(skb, true);
-   memset(>headers_start, 0,
-  offsetof(struct sk_buff, headers_end) -
-  offsetof(struct sk_buff, headers_start));
+   memset(>headers, 0, sizeof(skb->headers));
if (encapsulating) {
skb->l4_hash = l4_hash;
skb->sw_hash = sw_hash;
diff --git a/net/core/skbuff.c b/net/core/skbuff.c
index f9311762cc47..fd5ce57ccce6 100644
--- a/net/core/skbuff.c
+++ b/net/core/skbuff.c
@@ -991,12 +991,10 @@ void napi_consume_skb(struct sk_buff *skb, int budget)
 }
 EXPORT_SYMBOL(napi_consume_skb);
 
-/* Make sure a field is enclosed inside headers_start/headers_end section */
+/* Make sure a field is contained by headers group */
 #define CHECK_SKB_FIELD(field) \
-   BUILD_BUG_ON(offsetof(struct sk_buff, field) <  \
-offsetof(struct sk_buff, headers_start));  \
-   BUILD_BUG_ON(offsetof(struct sk_buff, field) >  \
-offsetof(struct sk_buff, headers_end));\
+   BUILD_BUG_ON(offsetof(struct sk_buff, field) != \
+offsetof(struct sk_buff, headers.field));  \
 
 static void __copy_skb_header(struct sk_buff *new, const struct sk_buff *old)
 {
@@ -1008,14 +1006,12 @@ static void __copy_skb_header(struct sk_buff *new, 
const struct sk_buff *old)
__skb_ext_copy(new, old);
__nf_copy(new, old, false);
 
-   /* Note : this field could be in headers_start/headers_end section
+   /* Note : this field could be in the headers group.
 * It is not yet because we do not want to have a 16 bit hole
 */
new->queue_mapping = old->queue_mapping;
 
-   memcpy(>headers_start, >headers_start,
-  offsetof(struct sk_buff, headers_end) -
-  offsetof(struct sk_buff, headers_start));
+   memcpy(>headers, >headers, sizeof(new->headers));
CHECK_SKB_FIELD(protocol);
CHECK_SKB_FIELD(csum);
CHECK_SKB_FIELD(hash);
diff --git a/include/linux/skbuff.h b/include/linux/skbuff.h
index 6bdb0db3e825..fee9041aa402 100644
--- a/include/linux/skbuff.h
+++ b/include/linux/skbuff.h
@@ -801,11 +801,10 @@ struct sk_buff {
__u8active_extensions;
 #endif
 
    /* fields enclosed in headers_start/headers_end are copied
 * using a single memcpy() in __copy_skb_header()
 */
-   /* private: */
-   __u32   headers_start[0];
+   struct_group(headers,
/* public: */
 
 /* if you move pkt_type around you also must adapt those constants */
@@ -922,8 +921,8 @@ struct sk_buff {
u64 kcov_handle;
 #endif
 
-   /* private: */
-   __u32   headers_end[0];
+   ); /* end headers group */
+
/* public: */
 
@@ -801,11 +801,10 @@ struct sk_buff {
 #endif
 
-   /* fields enclosed in headers_start/headers_end are copied
+   /* Fields enclosed in headers group are copied
 * using a single memcpy() in __copy_skb_header()
 */
/* These elements must be at the end, see alloc_skb() for details.  */
-- 
2.30.2